#! /bin/sh
#
PortID=linux_arm-x32;export PortID
#
#Everything Above Here Generated By bldinst.sh
#
#   Copyright 1988, 1989, 1990, 1991,    DNA Data Systems and Kevin English
#   Copyright 1991 - 1996                4C Associates
# 
#	@(#) - $Id: 4cinst.sh,v 3.12 2019/04/04 19:07:39 kjenglish Exp kjenglish $
#
#set -x
#
MyName=$0
HomeDir="$HOME_4C"
FCINSTHIST=/dev/tty
FCInfoChanged=No
TopDir=`pwd`
TmpDir=$TopDir/4cTmp$$
ODev=/dev/null
export ODev
trap catchhup 1
trap catchint 2
trap catchquit 3
trap catchterm 15
#
# Figure out the echo with & without newline command
#
eval print 1 >/dev/null 2>&1 &&
{
	PRINT=print
} ||
{
	PRINT=echo
}
$PRINT -n | grep -- "-n" &&
{
	PRINTN="$PRINT"
} ||
{
	PRINTN="$PRINT -n"
}
usage()
{
	$PRINT "Usage: $MyName [ options ... ]"
	$PRINT "	Options:"
	$PRINT "		-help"
	$PRINT "		-home <homedir>"
	exit 1
}
cleanup()
{
	if [ `basename "$TmpDir"` = "4cTmp$$" ]
	then
		rm -rf $TmpDir
	fi
}
instexit()
{
	ExitCode=$1
	if [ "$ExitCode" = "0" ]
	then
		$PRINT "	OK" >>$FCINSTHIST
	else
		$PRINT "	ERROR - $ExitCode" >>$FCINSTHIST
	fi
	$PRINT "Done - `date \"+%d %h %Y - %T\"`" >>$FCINSTHIST
	cleanup
	exit $ExitCode
}
catchhup()
{
	trap 1
	instexit 129
}
catchint()
{
	trap 2
	instexit 130
}
catchquit()
{
	trap 3
	instexit 131
}
catchterm()
{
	trap 15
	instexit 143
}
warn()
{
	$PRINT "	WARNING: $1"
	$PRINT "WARNING - $1" >> $TmpDir/Warn
	$PRINT "	WARNING - $1" >>$FCINSTHIST
}
fail()
{
	$PRINT "	FATAL ERROR: $1"
	$PRINT "FATAL ERROR - $1" >> $TmpDir/Fail
	$PRINT "	FATAL ERROR - $1" >> $FCINSTHIST
	instexit 1
}
#
# get yes/no answer
#
yn()
{
	Prompt=$1
	while $PRINTN "$Prompt? (y/n/Q) "
	do
		Answ=""
		read Answ
		if [ "$Answ" = "y" ]
		then
			return 0
		fi
		if [ "$Answ" = "n" ]
		then
			return 1
		fi
		if [ "$Answ" = "Q" ]
		then
			instexit 1
		fi
		$PRINT "Must Enter 'y', 'n', or 'Q' only"
	done
	return 1
}
#
# Only prompt if full install not specified
#
instyn()
{
	if [ "$FullInst" = "Yes" ]
	then
		return 0
	fi
	yn "$@"
}
#
# rewrite the FCINFO file if it changed
#
updatefcinfo()
{
	if [ $FCInfoChanged = "No" ]
	then
		chmod "$ROMode" "$FCINFO" 2>/dev/null
		chgrp "$FCGroup" "$FCINFO" 2>/dev/null
		chown "$FCOwner" "$FCINFO" 2>/dev/null
		return 0
	fi
	yn "Update 4C Info Configuration File" || return 1
	InfoDir=`dirname $FCINFO`
	if [ ! -w "$InfoDir" ]
	then
		fail "Dir $InfoDir is not writable - can't update 4C.info"
		return 1
	fi
	rm -f "$OLDFCINFO" || return 1
	mv -f "$FCINFO" "$OLDFCINFO" || return 1
	rm -f $FCINFO || return 1
	$PRINT "Owner	$FCOwner" >$FCINFO || return 1
	$PRINT "Group	$FCGroup" >>$FCINFO
	$PRINT "base_dir	$FCBaseDir" >>$FCINFO
	$PRINT "bin_dir	$FCBinDir" >>$FCINFO
	$PRINT "doc_dir	$FCDocDir" >>$FCINFO
	$PRINT "etc_dir	$FCEtcDir" >>$FCINFO
	$PRINT "lib_dir	$FCLibDir" >>$FCINFO
	$PRINT "samples_dir	$FCSampDir" >>$FCINFO
	$PRINT "system_dir	$FCSysDir" >>$FCINFO
	$PRINT "tutorial_dir	$FCTutDir" >>$FCINFO
	$PRINT "demo_dir	$FCDemoDir" >>$FCINFO
	$PRINT "image_dir	$FCImageDir" >>$FCINFO
	$PRINT "security	$FCSecurity" >>$FCINFO
#
	if [ -z "$OldFCOwner" ]
	then
		$PRINT "	$FCINFO Created" >>$FCINSTHIST
	else
		$PRINT "	$FCINFO Changed" >>$FCINSTHIST
		for FCVar in \
			FCOwner \
			FCGroup \
			FCBaseDir \
			FCBinDir \
			FCDocDir \
			FCEtcDir \
			FCLibDir \
			FCSampDir \
			FCSysDir \
			FCTutDir \
			FCDemoDir \
			FCImageDir \
			FCSecurity
		do
			eval "OldVal='$Old'$FCVar"
			eval "NewVal='$'$FCVar"
			Var=`expr $FCVar : 'FC\(.*\)'`
			if [ "$OldVal" != "$NewVal" ]
			then
				$PRINT "		$Var - $OldVal ==> $NewVal" >>$FCINSTHIST
			fi
		done
	fi
	chmod $ROMode $FCINFO 2>/dev/null
	chgrp "$FCGroup" "$FCINFO" 2>/dev/null
	chown "$FCOwner" "$FCINFO" 2>/dev/null
	return 0
}
verifyuser()
{
	grep -l "^$1:" /etc/passwd >$ODev 2>&1 && return 0
	$PRINT "Illegal User Name: $1"
	return 1
}
verifygroup()
{
	grep -l "^$1:" /etc/group >$ODev 2>&1 && return 0
	$PRINT "Illegal Group Name: $1"
	return 1
}
verifydir()
{
	if [ `dirname $1` = '.' ]
	then
		$PRINT "Full Pathname Required"
		return 1
	fi
	if [ ! -r "$TopDir/4cmkdirs.sh" ]
	then
		$PRINT "Non executable 4cmkdirs.sh"
		return 1
	fi
	/bin/sh $TopDir/4cmkdirs.sh "$1" || return 1
	if [ -z "$FCGroup" -o -z "$FCOwner" -o -z "$DirMode" ]
	then
		:
	else
		chgrp $FCGroup $1 2>/dev/null
		chown $FCOwner $1 2>/dev/null
		chmod $DirMode $1
	fi
	if [ ! -w  "$1" ]
	then
		$PRINT "Directory "$1" Must be writable for Install"
		return 1
	fi
	return 0
}
inpfcenv()
{
	Prompt="$1"
	eval CurVal='$'"{$2}"
	eval Old"$2"="$CurVal"
	DefVal="$CurVal"
	if [ -z "$DefVal" ]
	then
		DefVal="$3"
	fi
	while $PRINTN "$Prompt (or 'Q' to Quit): [$DefVal] "
	do
		answ=""
		read answ
		if [ -z "$answ" ]
		then
			answ="$DefVal"
		fi
		if [ ! -z "$answ" ]
		then
			break
		fi
	done
	if [ "$answ" = "Q" ]
	then
		instexit 1
	fi
#	$PRINT "$CurVal - $answ"
	eval New"$2"="$answ"
}
createinfo()
{
	$PRINT "Creating $FCINFO"
	(> $FCINFO) >$ODev 2>&1
	if [ ! -f "$FCINFO" ]
	then
		$PRINT "Cannot Create $FCINFO"
		instexit 1
	fi
	chmod 640 $FCINFO && return 0
	return 1
}
readinfo()
{
	if [ ! -f "$FCINFO" ]
	then
		createinfo && return 0 || return 1
	fi
	if [ ! -r "$FCINFO" ]
	then
		$PRINT "Cannot read file: $FCINFO"
		return 1
	fi
	exec 3< $FCINFO
	while read Reply 0<&3
	do
		set $Reply
		case $1 in
			    Owner)	FCOwner="$2"
						;;
			     Group)	FCGroup="$2"
						;;
			  base_dir) FCBaseDir="$2"
					;;
			   bin_dir) FCBinDir="$2"
					;;
			   doc_dir) FCDocDir="$2"
					;;
			   etc_dir) FCEtcDir="$2"
					;;
			   lib_dir) FCLibDir="$2"
					;;
			   samples_dir) FCSampDir="$2"
					;;
			  security) FCSecurity="$2"
					;;
			system_dir) FCSysDir="$2"
					;;
			tutorial_dir) FCTutDir="$2"
					;;
			demo_dir) FCDemoDir="$2"
					;;
			serial_dir) FCInfoHasSerialDir="Yes"
					;;
			image_dir) FCImageDir="$2"
					;;
			         *)	$PRINT "Illegal Line in $FCINFO $1"
					;;
		esac
	done
	exec 3<&-
	return 0
}
#
# Set or override all $FCINFO vars
#
modifyinfo()
{
#
# Security Type - (owner, group, none) - security must be first
#
	while :
	do
		inpfcenv "Enter 4C Security Type [owner|group|none] " 'FCSecurity' "group"
		if [ "$NewFCSecurity" = "owner" -o \
					"$NewFCSecurity" = "group" -o \
						"$NewFCSecurity" = "none" ]
		then
			break
		fi
		$PRINT "Security Type Must be [ owner | group | none ]"
		continue
	done
	if [ "$NewFCSecurity" != "$FCSecurity" ]
	then
		FCSecurity="$NewFCSecurity"
		FCInfoChanged=Yes
	fi
#
# The ETCEXMode allows read so we can get core files in the workdir
# if necessary.  Other binaries can be set readable when necessary.
#
#
	case $FCSecurity in
			owner)	EXMode="a-rwx,u=x"
					ETCEXMode="a-rwx,u=rx"
					DirMode="a-rwx,u=rwx"
					SHMode="a-rwx,u=rx"
					BootMode="a-rwx,u=rw"
					ROMode="a-rwx,u=r"
					RWRRMode="a-rwx,u=rw"
					;;
			none)   EXMode="a=x"
					ETCEXMode="a=rx"
					DirMode="a=rwx"
					SHMode="a=rx"
					BootMode="a=rw"
					ROMode="a=r"
					RWRRMode="a=r,u=rw"
					;;
			 group|*)	EXMode="a-rwx,g=x,u=x"
			 			ETCEXMode="a-rwx,g=rx,u=rx"
						DirMode="a-rwx,g=rwx,u=rwx"
						SHMode="a-rwx,g=rx,u=rx"
						BootMode="a-rwx,g=rw,u=rw"
						ROMode="a-rwx,g=r,u=r"
						RWRRMode="a-rwx,g=r,u=rw"
					;;
	esac
#
#	4C Owner
#
	while :
	do
		inpfcenv "Enter 4C Owner" 'FCOwner' "$LOGNAME"
		verifyuser $NewFCOwner || continue
		break
	done
	if [ "$NewFCOwner" != "$FCOwner" ]
	then
		FCOwner="$NewFCOwner"
		FCInfoChanged=Yes
	fi
#
#	4C Group
#
	while :
	do
		inpfcenv "Enter 4C Group" 'FCGroup' "$GRPNAME"
		verifygroup $NewFCGroup || continue
		break
	done
	if [ "$NewFCGroup" != "$FCGroup" ]
	then
		FCGroup="$NewFCGroup"
		FCInfoChanged=Yes
	fi
#
#	4C Master Directory
#
	while :
	do
		inpfcenv "Enter 4C Base Directory" 'FCBaseDir' "$HomeDir"
		verifydir $NewFCBaseDir || continue
		break
	done
	if [ "$NewFCBaseDir" != "$FCBaseDir" ]
	then
		FCBaseDir="$NewFCBaseDir"
		FCInfoChanged=Yes
	fi
#
#	4C Bin Directory
#
	while :
	do
		inpfcenv "Enter 4C Bin Directory" 'FCBinDir' "$FCBaseDir/bin"
		verifydir $NewFCBinDir || continue
		verifydir $NewFCBinDir/gnu || continue
		verifydir $NewFCBinDir/dbg || continue
		break
	done
	if [ "$NewFCBinDir" != "$FCBinDir" ]
	then
		FCBinDir="$NewFCBinDir"
		FCInfoChanged=Yes
	fi
#
#	4C Etc Directory
#
	while :
	do
		inpfcenv "Enter 4C Etc Directory" 'FCEtcDir' "$FCBaseDir/etc"
		verifydir $NewFCEtcDir || continue
		break
	done
	if [ "$NewFCEtcDir" != "$FCEtcDir" ]
	then
		FCEtcDir="$NewFCEtcDir"
		FCInfoChanged=Yes
	fi
#
#	4C Lib Directory
#
	while :
	do
		inpfcenv "Enter 4C Lib Directory" 'FCLibDir' "$FCBaseDir/lib"
		verifydir $NewFCLibDir || continue
		verifydir $NewFCLibDir/SC || continue
		verifydir $NewFCLibDir/misc || continue
		break
	done
	if [ "$NewFCLibDir" != "$FCLibDir" ]
	then
		FCLibDir="$NewFCLibDir"
		FCInfoChanged=Yes
	fi
#
#	4C Samples Directory
#
	while :
	do
		inpfcenv "Enter 4C Samples Directory" 'FCSampDir' "$FCBaseDir/samples"
		verifydir $NewFCSampDir || continue
		break
	done
	if [ "$NewFCSampDir" != "$FCSampDir" ]
	then
		FCSampDir="$NewFCSampDir"
		FCInfoChanged=Yes
	fi
#
#	4C Documentation Directory
#
	while :
	do
		inpfcenv "Enter 4C Documentation Directory" 'FCDocDir' "$FCBaseDir/doc"
		verifydir $NewFCDocDir || continue
		break
	done
	if [ "$NewFCDocDir" != "$FCDocDir" ]
	then
		FCDocDir="$NewFCDocDir"
		FCInfoChanged=Yes
	fi
#
#	4C System Bootstrap Directory
#
	while :
	do
		inpfcenv "Enter 4C System Program Directory" 'FCSysDir' "$FCBaseDir/system"
		verifydir $NewFCSysDir || continue
		break
	done
	if [ "$NewFCSysDir" != "$FCSysDir" ]
	then
		FCSysDir="$NewFCSysDir"
		FCInfoChanged=Yes
	fi
#
#	4C Tutorial Directory
#
	while :
	do
		inpfcenv "Enter 4C Tutorial Program Directory" 'FCTutDir' "$FCBaseDir/tutorial"
		verifydir $NewFCTutDir || continue
		break
	done
	if [ "$NewFCTutDir" != "$FCTutDir" ]
	then
		FCTutDir="$NewFCTutDir"
		FCInfoChanged=Yes
	fi
#
#	4C Demo Directory
#
	while :
	do
		inpfcenv "Enter 4C Demo Directory" 'FCDemoDir' "$FCBaseDir/demo"
		verifydir $NewFCDemoDir || continue
		break
	done
	if [ "$NewFCDemoDir" != "$FCDemoDir" ]
	then
		FCDemoDir="$NewFCDemoDir"
		FCInfoChanged=Yes
	fi
#
#	4C Image Directory
#
	while :
	do
		inpfcenv "Enter 4C Image Directory" 'FCImageDir' "$FCBaseDir/images"
		verifydir $NewFCImageDir || continue
		break
	done
	if [ "$NewFCImageDir" != "$FCImageDir" ]
	then
		FCImageDir="$NewFCImageDir"
		FCInfoChanged=Yes
	fi
	if [ "$FCInfoChanged" = "Yes" ]
	then
		return
	fi
	if [ "$FCInfoHasSerialDir" = "Yes" ]
	then
		yn "Removed Unneeded serial_dir entry from 4C.info" && FCInfoChanged=Yes
	fi
}
#
# Install binaries
#
BinInstall()
{
	if [ ! -d $1 ]
	then
		return 0
	fi
	verifydir $2
	(
	cd $1
	for GZFile in *.gz
	do
		if [ "$GZFile" = "*.gz" ]
		then
			break
		fi
		BinFile=`expr $GZFile : '\(.*\).gz'`
		$PRINTN "$GZFile ==> $2/$BinFile - "
		rm -f $2/$BinFile
		gzip -d -c $GZFile >$2/$BinFile ||
			{
				$PRINT "Failed";
				warn "Copy/Uncompress Failed - $BinFile";
				yn "Continue" || instexit 1
				continue;
			}
		if [ -f $BinFile.sum ]
		then
			rm -f $TmpDir/$BinFile$$
			sum $2/$BinFile |
				sed 's/ [^ ]*$//' >$TmpDir/$BinFile$$
			cmp -s $TmpDir/$BinFile$$ $BinFile.sum && 
					$PRINT "OK" ||
					{ $PRINT "Failed"; warn "Checksum Failed - $BinFile"; }
			rm -f $TmpDir/$BinFile$$
		fi
		chgrp $FCGroup $2/$BinFile 2>/dev/null
		chown $FCOwner $2/$BinFile 2>/dev/null
		if [ "$2" = "$FCEtcDir" ]
		then
			echo "$BinFile" | grep -q "4c" &&
								{ chmod $ETCEXMode $2/$BinFile; } ||
								{ chmod $EXMode $2/$BinFile; }
		elif [ "$2" = "$FCEtcDir/dbg" ]
		then
			echo "$BinFile" | grep -q "4c" &&
								{ chmod $ETCEXMode $2/$BinFile; } ||
								{ chmod $EXMode $2/$BinFile; }
		elif [ "$BinFile" = "4cRcs" ]
		then
			chmod $SHMode $2/$BinFile
		elif [ "$BinFile" = "4cSccs" ]
		then
			chmod $SHMode $2/$BinFile
		elif [ "$BinFile" = "4cShell" ]
		then
			chmod $SHMode $2/$BinFile
		else
			file $2/$BinFile | egrep -i "text|shell" >$ODev 2>&1 && 
								{ chmod $SHMode $2/$BinFile; } ||
								{ chmod $EXMode $2/$BinFile; }
		fi
	done
	) || exit $?
}
#
# Install All Files in Dir Skipping or Recursing into subdirectories
#
DirInstall()
{
	verifydir $2
	if [ ! -d $1 ]
	then
		return 0
	fi
	MyMode=$3
	if [ "$4" = "Recurse" ]
	then
		Recurse="Yes"
	fi
	(
	cd $1
#
# Subdirs first
#
	if [ "$Recurse" = "Yes" ]
	then
		for FileName in *
		do
			if [ -d "$FileName" ]
			then
				DirInstall "$FileName" "$2/$FileName" "$MyMode" "Recurse"
			fi
		done
	fi
	for GZFile in *.gz
	do
		if [ "$GZFile" = "*.gz" ]
		then
			break
		fi
		BootFile=`expr $GZFile : '\(.*\).gz'`
		$PRINTN "$GZFile ==> $2/$BootFile - "
		rm -f $2/$BootFile
		gzip -d -c $GZFile >$2/$BootFile ||
			{
				$PRINT "Failed";
				warn "Copy/Uncompress Failed - $BootFile";
				yn "Continue" || instexit 1
				continue;
			}
		if [ -f $BootFile.sum ]
		then
			rm -f $TmpDir/$BootFile$$
			sum $2/$BootFile |
				sed 's/ [^ ]*$//' >$TmpDir/$BootFile$$
			cmp -s $TmpDir/$BootFile$$ $BootFile.sum && 
					$PRINT "OK" ||
					{ $PRINT "Failed"; warn "Checksum Failed - $BootFile"; }
			rm -f $TmpDir/$BootFile$$
		fi
		chgrp $FCGroup $2/$BootFile 2>/dev/null
		chown $FCOwner $2/$BootFile 2>/dev/null
		chmod $MyMode $2/$BootFile
	done
	) || exit $?
}
#
# Install One File Only
#
FileInstall()
{
	FromFile="$1"
	FileName=`basename $FromFile`
	GZFile="$FromFile.gz"
	ToFile="$2"
	MyMode="$3"
	$PRINTN "$GZFile ==> $ToFile - "
	rm -f $ToFile
	gzip -d -c $GZFile >$ToFile ||
			{
				$PRINT "Failed";
				warn "Copy/Uncompress Failed - $ToFile";
				yn "Continue" || instexit 1
				continue;
			}
	if [ -f $FromFile.sum ]
	then
		rm -f $TmpDir/$FileName$$
		sum $ToFile |
			sed 's/ [^ ]*$//' >$TmpDir/$FileName$$
		cmp -s $TmpDir/$FileName$$ $FromFile.sum && 
				$PRINT "OK" ||
				{ $PRINT "Failed"; warn "Checksum Failed - $FileName"; }
		rm -f $TmpDir/$FileName$$
	fi
	chgrp $FCGroup $ToFile 2>/dev/null
	chown $FCOwner $ToFile 2>/dev/null
	chmod $MyMode $ToFile
}
#
# Generate script files if they do not exist in HOME_4C
#
genscriptfiles()
{
#
# Always create and overwrite config/default
# Then copy if necessary
#
	FN="$FCConfigDir/default/4Crc"
	$PRINT "Generating $FN"
	rm -f "$FN"
	$PRINT "HOME_4C=$HomeDir" >"$FN"
	$PRINT "echo \"\$PATH\" | grep -q \"\$HOME_4C/bin:\" || {" >>"$FN"
	$PRINT "  PATH=\$HOME_4C/bin:\$PATH" >>"$FN"
	$PRINT "}" >>"$FN"
	$PRINT "JISCONFIG=\$HOME_4C/config/jisamconfig" >>"$FN"
	$PRINT "#JISCONFIG64=\$HOME_4C/config/jisamconfig64" >>"$FN"
	$PRINT "XLSYSTEM=\$HOME_4C/system" >>"$FN"
	$PRINT "XLCONFIG=\$HOME_4C/config/4CConfig" >>"$FN"
	$PRINT "XLLIB=\$HOME_4C/lib" >>"$FN"
	$PRINT "_4CSRVRCONFIG=\$HOME_4C/config/4CSrvrConfig" >>"$FN"
	$PRINT "	export HOME_4C PATH JISCONFIG JISCONFIG64 XLCONFIG \
XLLIB _4CSRVRCONFIG" >>"$FN"
	$PRINT "	export XLSYSTEM XLPROG XLEXP XLDATA XLAPP" >>"$FN"
	if [ ! -f "$HomeDir/4Crc" ]
	then
		cp "$FN" "$HomeDir/4Crc"
		chmod $BootMode $HomeDir/4Crc
		chgrp "$FCGroup" $HomeDir/4Crc
		chown "$FCOwner" $HomeDir/4Crc
	fi
	FN="$FCConfigDir/default/run4c.sh"
	$PRINT "Generating $FN"
	rm -f "$FN"
	$PRINT "#!/bin/sh" >"$FN"
	$PRINT "#" >> "$FN"
	$PRINT "# 4C Daemon Startup Script" >> "$FN"
	$PRINT "#" >> "$FN"
	$PRINT "echo \"\"" >>"$FN"
	$PRINT "echo \"Starting 4C Server Daemon\"" >>"$FN"
	$PRINT "echo \"\"" >>"$FN"
	$PRINT "HOME_4C=$HomeDir" >>"$FN"
	$PRINT ". \$HOME_4C/4Crc" >>"$FN"
	$PRINT "FCSHELL=/bin/sh" >>"$FN"
	$PRINT "	export FCSHELL" >>"$FN"
	$PRINT "unset FC_APPDATA" >>"$FN"
	$PRINT "unset FC_DOCUMENTS" >>"$FN"
	if [ "$PortID" = linux_386 ]
	then
		$PRINT "ulimit -c unlimited" >>"$FN"
	fi
	$PRINT "cd \$HOME_4C/etc" >>"$FN"
	$PRINT "./4csrvrd -S >/dev/null 2>&1 </dev/null && { echo Already Running; exit 1; }" >>"$FN"
	$PRINT "rm -f \$HOME_4C/tmp/XL*" >>"$FN"
	$PRINT "find \$HOME_4C/tmp -name core.\* -mtime +7 -exec rm -f {} \\;" >>"$FN"
	$PRINT "./4csrvrd >/dev/null 2>&1 </dev/null" >>"$FN"
	$PRINT "sleep 3" >>"$FN"
	$PRINT "./4csrvrd -S" >>"$FN"
	$PRINT "echo \"\"" >>"$FN"
	$PRINT "#" >>"$FN"
	$PRINT "# Start background clients if any" >>"$FN"
	$PRINT "#" >>"$FN"
	$PRINT "if [ -e \$HOME_4C/scripts/run4c-bgcl.sh ]" >>"$FN"
	$PRINT "then" >>"$FN"
	$PRINT "  /bin/sh \$HOME_4C/scripts/run4c-bgcl.sh" >>"$FN"
	$PRINT "fi" >>"$FN"
	if [ ! -f "$HomeDir/scripts/run4c.sh" ]
	then
		cp "$FN" "$HomeDir/scripts/run4c.sh"
		chmod $BootMode $HomeDir/scripts/run4c.sh
		chgrp "$FCGroup" $HomeDir/scripts/run4c.sh
		chown "$FCOwner" $HomeDir/scripts/run4c.sh
	fi
	FN="$FCConfigDir/default/stop4c.sh"
	$PRINT "Generating $FN"
	rm -f "$FN"
	$PRINT "#!/bin/sh" >"$FN"
	$PRINT "#" >> "$FN"
	$PRINT "# 4C Daemon Stop Script" >> "$FN"
	$PRINT "#" >> "$FN"
	$PRINT "echo \"\"" >>"$FN"
	$PRINT "echo \"Stopping 4C Server Daemon\"" >>"$FN"
	$PRINT "echo \"\"" >>"$FN"
	$PRINT "HOME_4C=$HomeDir" >>"$FN"
	$PRINT "_4CSRVRCONFIG=\$HOME_4C/config/4CSrvrConfig" >>"$FN"
	$PRINT "	export HOME_4C _4CSRVRCONFIG" >>"$FN"
	$PRINT "cd \$HOME_4C/etc" >>"$FN"
	$PRINT "./4csrvrd -K" >>"$FN"
	$PRINT "sleep 3" >>"$FN"
	$PRINT "./4csrvrd -S" >>"$FN"
	$PRINT "echo \"\"" >>"$FN"
	if [ ! -f "$HomeDir/scripts/stop4c.sh" ]
	then
		cp "$FN" "$HomeDir/scripts/stop4c.sh"
		chmod $BootMode $HomeDir/scripts/stop4c.sh
		chgrp "$FCGroup" $HomeDir/scripts/stop4c.sh
		chown "$FCOwner" $HomeDir/scripts/stop4c.sh
	fi
	FN="$FCConfigDir/default/restart4c.sh"
	$PRINT "Generating $FN"
	rm -f "$FN"
	$PRINT "#!/bin/sh" >"$FN"
	$PRINT "#" >> "$FN"
	$PRINT "# 4C Daemon Restart Script" >> "$FN"
	$PRINT "#" >> "$FN"
	$PRINT "echo \"\"" >>"$FN"
	$PRINT "echo \"Restarting 4C Server Daemon\"" >>"$FN"
	$PRINT "echo \"\"" >>"$FN"
	$PRINT "HOME_4C=$HomeDir" >>"$FN"
	$PRINT "_4CSRVRCONFIG=\$HOME_4C/config/4CSrvrConfig" >>"$FN"
	$PRINT "	export HOME_4C _4CSRVRCONFIG" >>"$FN"
	$PRINT "cd \$HOME_4C/etc" >>"$FN"
	$PRINT "./4csrvrd -S >/dev/null 2>&1 </dev/null || { echo Not Running; exit 1; }" >>"$FN"
	$PRINT "./4csrvrd -R >/dev/null 2>&1 </dev/null" >>"$FN"
	$PRINT "sleep 3" >>"$FN"
	$PRINT "./4csrvrd -S" >>"$FN"
	$PRINT "echo \"\"" >>"$FN"
	if [ ! -f "$HomeDir/scripts/restart4c.sh" ]
	then
		cp "$FN" "$HomeDir/scripts/restart4c.sh"
		chmod $BootMode $HomeDir/scripts/restart4c.sh
		chgrp "$FCGroup" $HomeDir/scripts/restart4c.sh
		chown "$FCOwner" $HomeDir/scripts/restart4c.sh
	fi
	FN="$FCConfigDir/default/4CSrvrConfig"
	$PRINT "Generating $FN"
	rm -f "$FN"
	$PRINT "LogFile 	$HomeDir/log/4c.log" >"$FN"
	$PRINT "Connect		Type=Socket,Port=1412" >>"$FN"
	$PRINT "WorkDir		$HomeDir/tmp" >>"$FN"
	$PRINT "ServerDir	$HomeDir/etc" >>"$FN"
	$PRINT "DpyAlarm	10" >>"$FN"
	$PRINT "ClUpdAlarm	10" >>"$FN"
#
# These should only be set by 4Crc
#
#	$PRINT "Env			HOME_4C		$HomeDir" >>"$FN"
#	$PRINT "Env			JISCONFIG $HomeDir/config/jisamconfig" >>"$FN"
#	$PRINT "#Env			JISCONFIG64 $HomeDir/config/jisamconfig64" >>"$FN"
#	$PRINT "Env			XLCONFIG $HomeDir/config/4CConfig" >>"$FN"
#	$PRINT "Env			XLLIB $HomeDir/lib" >>"$FN"
#	$PRINT "Env			system_dir $HomeDir/system" >>"$FN"
#	$PRINT "Env			prog_lst $HomeDir/system" >>"$FN"
	$PRINT "Env			XLMODETOGGLE ON" >>"$FN"
	$PRINT "Env			XLMODECONT ON" >>"$FN"
	$PRINT "#" >>"$FN"
	$PRINT "# Set LogonSecurity to one of Default/PAM/MD5" >>"$FN"
	$PRINT "#Default/PAM/MD5" >>"$FN"
	$PRINT "#LogonSecurity		Default" >>"$FN"
	$PRINT "#" >>"$FN"
	$PRINT "# Allow some clients programs to reconnect when connection is lost" >>"$FN"
	$PRINT "#" >>"$FN"
	$PRINT "AllowReconnect		True" >>"$FN"
	$PRINT "#" >>"$FN"
	$PRINT "# allow 4csrvr to continue working when client connection is lost" >>"$FN"
	$PRINT "#" >>"$FN"
	$PRINT "ContinueWork		True" >>"$FN"
	$PRINT "Application 4cSys" >>"$FN"
	$PRINT "	Desc=4CBootstraps" >>"$FN"
	$PRINT "	Prog=sys.menu.tv" >>"$FN"
	$PRINT "	Group=StartGroup" >>"$FN"
	$PRINT "	prog_lst=$HomeDir/system" >>"$FN"
	$PRINT "	data_lst=$HomeDir/system" >>"$FN"
	$PRINT "	Icon=system/4cboot2.ico" >>"$FN"
	$PRINT "Application 4cDemo" >>"$FN"
	$PRINT "	Desc=4CDemo" >>"$FN"
	$PRINT "	Prog=demo.main.s" >>"$FN"
	$PRINT "	Group=StartGroup" >>"$FN"
	$PRINT "	prog_lst=$HomeDir/demo:$HomeDir/system" >>"$FN"
	$PRINT "	data_lst=$HomeDir/demo/DATA:$HomeDir/demo:$HomeDir/system" >>"$FN"
	if [ ! -f "$HomeDir/config/4CSrvrConfig" ]
	then
		cp "$FN" "$HomeDir/config/4CSrvrConfig"
		chmod $RWRRMode $HomeDir/config/4CSrvrConfig
		chgrp "$FCGroup" $HomeDir/config/4CSrvrConfig
		chown "$FCOwner" $HomeDir/config/4CSrvrConfig
	fi
	if [ ! -f "$HomeDir/config/jisamconfig" ]
	then
		$PRINT "Generating jisamconfig"
		JISCONFIG=$HomeDir/config/jisamconfig
		export JISCONFIG
		$HomeDir/bin/isconfig -c -f 1000 -b 200 -u 500 -k 20 -a 50 -l 20 \
			-n 2 -B 6 -F 100 >/dev/null
		chmod $RWRRMode $HomeDir/config/jisamconfig
		chgrp "$FCGroup" $HomeDir/config/jisamconfig
		chown "$FCOwner" $HomeDir/config/jisamconfig
	fi
	if [ ! -f "$HomeDir/config/jisamconfig.lk" ]
	then
		>$HomeDir/config/jisamconfig.lk
		chmod $BootMode $HomeDir/config/jisamconfig.lk
		chgrp "$FCGroup" $HomeDir/config/jisamconfig.lk
		chown "$FCOwner" $HomeDir/config/jisamconfig.lk
	fi
	if [ ! -f "$HomeDir/config/jisamconfig64" ]
	then
		$PRINT "Generating jisamconfig64"
		JISCONFIG64=$HomeDir/config/jisamconfig64
		export JISCONFIG64
		$HomeDir/bin/isconfig64 -c -f 1000 -b 200 -u 500 -k 20 -a 50 -l 20 \
			-n 2 -B 6 -F 100 >/dev/null
		chmod $RWRRMode $HomeDir/config/jisamconfig64
		chgrp "$FCGroup" $HomeDir/config/jisamconfig64
		chown "$FCOwner" $HomeDir/config/jisamconfig64
	fi
	if [ ! -f "$HomeDir/config/jisamconfig64.lk" ]
	then
		>$HomeDir/config/jisamconfig64.lk
		chmod $BootMode $HomeDir/config/jisamconfig64.lk
		chgrp "$FCGroup" $HomeDir/config/jisamconfig64.lk
		chown "$FCOwner" $HomeDir/config/jisamconfig64.lk
	fi
	FN="$FCConfigDir/default/4CConfig"
	$PRINT "Generating $FN"
	rm -f "$FN"
	JISAMADDR=2000
	JISAM64ADDR=2004
	SYSADDR=2008
	if [ "$PortID" = sco_u386 ]
	then
		JISAMADDR=2000
		JISAM64ADDR=2004
		SYSADDR=2008
	fi
	if [ "$PortID" = linux_386 ]
	then
		JISAMADDR=2000
		JISAM64ADDR=2004
		SYSADDR=2008
	fi
	if [ "$PortID" = rs6000 ]
	then
		JISAMADDR=768
		JISAM64ADDR=1024
		SYSADDR=1280
	fi
	$PRINT "Env HOME_4C $HomeDir" >"$FN"
	$PRINT "Files 1000 1000" >>"$FN"
	$PRINT "Programs 200 200" >>"$FN"
	$PRINT "SysMemory 32768 1" >>"$FN"
	if [ "$PortID" = hp9000 ]
	then
		$PRINT "#Can't Set JISAMAddr on HP-UX" >>"$FN"
		$PRINT "#Can't Set JISAM64Addr on HP-UX" >>"$FN"
		$PRINT "#Can't Set SysAddr on HP-UX" >>"$FN"
	else
		$PRINT "JISAMAddr $JISAMADDR" >>"$FN"
		$PRINT "JISAM64Addr $JISAM64ADDR" >>"$FN"
		$PRINT "SysAddr $SYSADDR" >>"$FN"
	fi
	$PRINT "Users 100" >>"$FN"
	$PRINT "FCPort 1412" >>"$FN"
	$PRINT "ErrLogFile $HomeDir/log/4cLog" >>"$FN"
	$PRINT "ErrLog all" >>"$FN"
	$PRINT "tmp_dir $HomeDir/tmp" >>"$FN"
	$PRINT "SvTimeOut	7200" >>"$FN"
	$PRINT "ClTimeOut	7200" >>"$FN"
	$PRINT "#MultiLockOK" >>"$FN"
	if [ ! -f "$HomeDir/config/4CConfig" ]
	then
		cp "$FN" "$HomeDir/config/4CConfig"
		chmod $RWRRMode $HomeDir/config/4CConfig
		chgrp "$FCGroup" $HomeDir/config/4CConfig
		chown "$FCOwner" $HomeDir/config/4CConfig
	fi
	if [ ! -f "$HomeDir/config/4CConfig.lk" ]
	then
		>$HomeDir/config/4CConfig.lk
		chmod $BootMode $HomeDir/config/4CConfig.lk
		chgrp "$FCGroup" $HomeDir/config/4CConfig.lk
		chown "$FCOwner" $HomeDir/config/4CConfig.lk
	fi
	FN="$FCConfigDir/default/check4c.sh"
	$PRINT "Generating $FN"
	rm -f "$FN"
	$PRINT "#!/bin/sh" >"$FN"
	$PRINT "_4CLOGFILE=\"$HomeDir/log/4cLog\"" >>"$FN"
	$PRINT "	export _4CLOGFILE" >>"$FN"
	$PRINT "WORKDIR=\"$HomeDir/tmp\"" >>"$FN"
	$PRINT "	export WORKDIR" >>"$FN"
	$PRINT "echo \"  4C Started Procs:\" \`grep -c StartProc \"\$_4CLOGFILE\"\`" >>"$FN"
	$PRINT "echo \" 4C Finished Procs:\" \`grep -c EndProc \"\$_4CLOGFILE\"\`" >>"$FN"
	$PRINT "echo \"Current Isam Procs:\" \`isview | grep -c \"Process Id\"\`" >>"$FN"
	$PRINT "echo \"            4csrvr: \" \`ps -el | grep -c \"4csrvr$\"\`" >>"$FN"
	$PRINT "echo \"           4cbuild: \" \`ps -el | grep -c \"4cbuild$\"\`" >>"$FN"
	$PRINT "echo \"              4cfs: \" \`ps -el | grep -c \"4cfs$\"\`" >>"$FN"
	$PRINT "echo \"             4cprt: \" \`ps -el | grep -c \"4cprt$\"\`" >>"$FN"
	$PRINT "echo \"JIsam \" \`isview | grep LRU\`" >>"$FN"
	$PRINT "COREFOUND=\"No\"" >>"$FN"
	$PRINT "if [ -f \"\$WORKDIR/core\" ]" >>"$FN"
	$PRINT "then" >>"$FN"
	$PRINT "	echo \"Core File Found\"" >>"$FN"
	$PRINT "	ls -ls \"\$WORKDIR/core\"" >>"$FN"
	$PRINT "	COREFOUND=\"Yes\"" >>"$FN"
	$PRINT "fi" >>"$FN"
	$PRINT "if [ \"\$COREFOUND\" = \"No\" ]" >>"$FN"
	$PRINT "then" >>"$FN"
	$PRINT "	echo \"No Core Files\"" >>"$FN"
	$PRINT "fi" >>"$FN"
	$PRINT "cd $HomeDir/tmp" >>"$FN"
	$PRINT "for X in \`ls XL* | sed '" >>"$FN"
	$PRINT "s/XLUT_..0*//" >>"$FN"
	$PRINT "s/XL_..0*//' | sort | uniq\`" >>"$FN"
	$PRINT "do" >> "$FN"
	$PRINT "	xlview | grep -q \$X || echo \"\$X Not Found\"" >>"$FN"
	$PRINT "done" >>"$FN"
	if [ ! -f "$HomeDir/scripts/check4c.sh" ]
	then
		cp "$FN" "$HomeDir/scripts/check4c.sh"
		chmod $BootMode $HomeDir/scripts/check4c.sh
		chgrp "$FCGroup" $HomeDir/scripts/check4c.sh
		chown "$FCOwner" $HomeDir/scripts/check4c.sh
	fi
	FN="$FCConfigDir/default/4cshutdown.sh"
	$PRINT "Generating $FN"
	rm -f "$FN"
	$PRINT "#!/bin/sh" >"$FN"
	$PRINT "#" >>"$FN"
	$PRINT "# 4C Shutdown Script" >>"$FN"
	$PRINT "#" >>"$FN"
	$PRINT "HOME_4C=$HomeDir" >>"$FN"
	$PRINT "export HOME_4C" >>"$FN"
	$PRINT "cd \$HOME_4C" >>"$FN"
	$PRINT ". ./4Crc" >>"$FN"
	$PRINT "/bin/sh \$HOME_4C/scripts/stop4c.sh" >>"$FN"
	$PRINT "echo \"Shutting Down 4C Servers\"" >>"$FN"
	$PRINT "4cshutdown -w 60 -f 60" >>"$FN"
	$PRINT "exit 0" >>"$FN"
	if [ ! -f "$HomeDir/scripts/4cshutdown.sh" ]
	then
		cp "$FN" "$HomeDir/scripts/4cshutdown.sh"
		chmod $BootMode $HomeDir/scripts/4cshutdown.sh
		chgrp "$FCGroup" $HomeDir/scripts/4cshutdown.sh
		chown "$FCOwner" $HomeDir/scripts/4cshutdown.sh
	fi
}
#
# Process args
#
while test "$#" != "0"
do
	arg=$1
	shift 1
	case $arg in
		-help|-h|-H)	usage ;;
		-home)	if test "$#" = 0
				then
					$PRINT "Must Specify Directory Name with -home"
					usage
				fi
				HomeDir=$1;shift 1;;
		*)		echo "Illegal arg: $arg"
				usage;;
	esac
done
if [ -z "$LOGNAME" ]
then
	for X in `who am i`
	do
		LOGNAME="$X"
		break
	done
fi
#
$PRINT "JISAM64 files created by any 4csrvr version earlier than 6.0 must be converted"
yn "Continue" || instexit 1
if [ -z "$HomeDir" ]
then
	HOME_4C="/usr/4c"
	$PRINT "4C Home Directory Not Set or Specified"
	yn "OK to use \"$HOME_4C\" " ||
		{ echo "Please set HOME_4C or use -home option";
			usage; }
	HomeDir=$HOME_4C
else
	if [ `dirname $HomeDir` = "." ]
	then
		$PRINT "Invalid Home Dir for 4C: $HomeDir"
		$PRINT "Full Pathname required for 4C Home Directory"
		usage
	fi
fi
verifydir "$HomeDir" || exit 1
FCINFO="$HomeDir/4C.info"
OLDFCINFO="$HomeDir/O4C.info"
if [ -z "$PortID" ]
then
	$PRINT "Must Set PortID"
	instexit 1
fi
if [ -d $TmpDir ]
then
	rm -rf $TmpDir
fi
mkdir $TmpDir  || instexit 1
if [ -z "$GRPNAME" ]
then
	>$TmpDir/X$$
	GRPNAME=`ls -l $TmpDir/X$$ |
	awk '{ \
			print $4; \
	}'`
fi
PATH=$TopDir:$PATH
	export PATH
readinfo || instexit 1
modifyinfo
FCConfigDir="$FCBaseDir/config"
FCScriptsDir="$FCBaseDir/scripts"
FCINSTHIST=$FCEtcDir/4Cinst.hist
date "+%d %h %Y - %T - $LOGNAME" >>$FCINSTHIST
#
# Re verify homedir to set permissions
#
verifydir "$HomeDir" || instexit 1
#
# Make sure config/default exist
#
verifydir "$FCConfigDir" || instexit 1
verifydir "$FCConfigDir/default" || instexit 1
#
# Create scripts directory if necessary
#
verifydir "$FCScriptsDir" || instexit 1
#
updatefcinfo || instexit 1
yn "Full Install" && FullInst="Yes"
#
#
instyn "Install 4C Binaries" &&
{
	$PRINT "	Installing Binaries" >>$FCINSTHIST
	if [ -f "$TopDir/4Cbin.ver" ]
	then
		cat $TopDir/4Cbin.ver >>$FCINSTHIST
	fi
	$PRINT "Copying Binaries"
	BinInstall $TopDir/bin/gnu $FCBinDir/gnu
	BinInstall $TopDir/bin/dbg $FCBinDir/dbg
	BinInstall $TopDir/bin $FCBinDir
#	chown root $FCBinDir/iskiller
#	chmod +s $FCBinDir/iskiller
}
instyn "Install 4C Bootstraps" &&
{
	$PRINT "	Installing Bootstraps" >>$FCINSTHIST
	$PRINT "Copying Bootstraps"
	DirInstall $TopDir/system $FCSysDir $BootMode
	for FileProg in FILE PROGRAM SL
	do
		(cd $TopDir/system/$FileProg;
		FileProgDir=`pwd`
		verifydir $FCSysDir/$FileProg
		for DirName in ?
		do
			verifydir $FCSysDir/$FileProg/$DirName
			verifydir $FCSysDir/$FileProg/$DirName/RCS
			cd $FileProgDir/$DirName
			if [ -d RCS ]
			then
				cd RCS
				DirInstall `pwd` $FCSysDir/$FileProg/$DirName/RCS $ROMode
			fi
		done) || exit $?
	done
	DirInstall $TopDir/system/security $FCSysDir/security $BootMode
	DirInstall $TopDir/system/lib $FCSysDir/lib $ROMode
	for X in $FCSysDir/lib/*.so
	do
		if [ -f $X ]
		then
			chmod "$SHMode" "$X"
		fi
	done
	for X in $FCSysDir/lib/*.sl
	do
		if [ -f $X ]
		then
			chmod "$SHMode" "$X"
		fi
	done
	for FCLibFileGZ in $TopDir/lib/*.xl.gz
	do
		FCLibFile=`expr $FCLibFileGZ : '\(.*\)\.gz$'`
		FCSysFile=$FCSysDir/`basename $FCLibFile`
		if [ ! -f "$FCSysFile" ]
		then
			FileInstall $FCLibFile $FCSysFile $BootMode
		fi
	done
}
instyn "Install 4C Tutorial" &&
{
	$PRINT "	Installing Tutorial" >>$FCINSTHIST
	$PRINT "Copying Tutorial"
	DirInstall $TopDir/tutorial $FCTutDir $BootMode
	DirInstall $TopDir/tutorial/DATA $FCTutDir/DATA $BootMode
#	for FileProg in FILE PROGRAM SL
#	do
#		(cd $TopDir/tutorial/$FileProg;
#		FileProgDir=`pwd`
#		verifydir $FCTutDir/$FileProg
#		for DirName in ?
#		do
#			verifydir $FCTutDir/$FileProg/$DirName
#			verifydir $FCTutDir/$FileProg/$DirName/RCS
#			cd $FileProgDir/$DirName
#			if [ -d RCS ]
#			then
#				cd RCS
#				DirInstall `pwd` $FCTutDir/$FileProg/$DirName/RCS $ROMode
#			fi
#		done) || exit $?
#	done
}
instyn "Install 4C Demo" &&
{
	$PRINT "	Installing Demo" >>$FCINSTHIST
	$PRINT "Copying Demo"
	DirInstall $TopDir/demo $FCDemoDir $BootMode
	DirInstall $TopDir/demo/DATA $FCDemoDir/DATA $BootMode
	DirInstall $TopDir/demo/Keys $FCDemoDir/Keys $ROMode
	for FileProg in FILE PROGRAM SL
	do
		(cd $TopDir/demo/$FileProg;
		FileProgDir=`pwd`
		verifydir $FCDemoDir/$FileProg
		for DirName in ?
		do
			verifydir $FCDemoDir/$FileProg/$DirName
			verifydir $FCDemoDir/$FileProg/$DirName/RCS
			cd $FileProgDir/$DirName
			if [ -d RCS ]
			then
				cd RCS
				DirInstall `pwd` $FCDemoDir/$FileProg/$DirName/RCS $ROMode
			fi
		done) || exit $?
	done
}
instyn "Install Etc Files" &&
{
	$PRINT "	Installing Etc" >>$FCINSTHIST
	rm -f $FCEtcDir/4cstat $FCEtcDir/dbg/4cstat
	$PRINT "Copying Etc Files"
	BinInstall $TopDir/etc $FCEtcDir
	BinInstall $TopDir/etc/dbg $FCEtcDir/dbg
	chown root $FCEtcDir/4csrvrd
	chmod 500 $FCEtcDir/4csrvrd
	chown root $FCEtcDir/dbg/4csrvrd
	chmod 500 $FCEtcDir/dbg/4csrvrd
}
instyn "Install Lib Files" &&
{
	$PRINT "	Installing Lib" >>$FCINSTHIST
	$PRINT "Copying Lib Files"
	DirInstall $TopDir/lib $FCLibDir $ROMode
	DirInstall $TopDir/lib/SC $FCLibDir/SC $SHMode
	DirInstall $TopDir/lib/misc $FCLibDir/misc $SHMode
	ClientsDir=$FCLibDir/clients
	/bin/sh $TopDir/4cmkdirs.sh $ClientsDir
	if [ -z "$FCGroup" -o -z "$FCOwner" -o -z "$DirMode" ]
	then
		:
	else
		chgrp $FCGroup $ClientsDir 2>/dev/null
		chown $FCOwner $ClientsDir 2>/dev/null
		chmod $DirMode $ClientsDir
	fi
#	DirInstall $TopDir/lib/app-defaults $FCLibDir/app-defaults $ROMode
#	DirInstall $TopDir/lib/clients $FCLibDir/clients $ROMode
#	rm -f $FCLibDir/clients/*
#	for CLFILEGZ in $TopDir/lib/clients/*.gz
#	do
#		CLFILE=`expr $CLFILEGZ : '\(.*\)\.gz$'`
#		FileInstall $CLFILE $FCLibDir/clients/`basename $CLFILE` $ROMode
#	done
#	cp $TopDir/lib/.4csrclib.tar.gz $FCLibDir/.4csrclib.tar.gz
	(cd $FCLibDir
	for SHFile in *.sh
	do
		if [ "$SHFile" = "*.sh" ]
		then
			break
		fi
		chmod $SHMode $SHFile
	done)
#
# Set up Symbolic Links to Shared Libraries
#
	(cd $FCLibDir
	if [ -f rellib4ccrypto.so ]
	then
		chmod $SHMode rellib4ccrypto.so
		rm -f lib4ccrypto.so
		ln -s rellib4ccrypto.so lib4ccrypto.so
	fi
	if [ -f rellib4ccrypto.sl ]
	then
		chmod $SHMode rellib4ccrypto.sl
		rm -f lib4ccrypto.sl
		ln -s rellib4ccrypto.sl lib4ccrypto.sl
	fi
	if [ -f rellibjisam64.so ]
	then
		chmod $SHMode rellibjisam64.so
		rm -f libjisam64.so
		ln -s rellibjisam64.so libjisam64.so
	fi
	if [ -f rellibjisam64.sl ]
	then
		chmod $SHMode rellibjisam64.sl
		rm -f libjisam64.sl
		ln -s rellibjisam64.sl libjisam64.sl
	fi
	if [ -f rellibjisam32.so ]
	then
		chmod $SHMode rellibjisam32.so
		rm -f libjisam32.so
		ln -s rellibjisam32.so libjisam32.so
	fi
	if [ -f rellibjisam32.sl ]
	then
		chmod $SHMode rellibjisam32.sl
		rm -f libjisam32.sl
		ln -s rellibjisam32.sl libjisam32.sl
	fi
	)
#
# End of Shared Lib setup
#
}
instyn "Install Image Files" &&
{
	$PRINT "	Installing Images" >>$FCINSTHIST
	$PRINT "Copying Image Files"
	DirInstall $TopDir/images $FCImageDir $ROMode "Recurse"
}
instyn "Install Samples Files" &&
{
	$PRINT "	Installing Samples" >>$FCINSTHIST
	$PRINT "Copying Samples"
	DirInstall $TopDir/samples $FCSampDir $ROMode "Recurse"
}
instyn "Install Doc Files" &&
{
	$PRINT "	Installing Documentation" >>$FCINSTHIST
	$PRINT "Copying Documentation"
	DirInstall $TopDir/doc $FCDocDir $ROMode "Recurse"
}
instyn "Install Web4CEtc Files" &&
{
	$PRINT "	Installing Web4CEtc" >>$FCINSTHIST
	$PRINT "Copying Web4CEtc Files"
	BinInstall $TopDir/Web4C/etc $HomeDir/Web4C/etc
	BinInstall $TopDir/Web4C/etc/dbg $HomeDir/Web4C/etc/dbg
	chown root $HomeDir/Web4C/etc/4creqd
	chmod 500 $HomeDir/Web4C/etc/4creqd
	chown root $HomeDir/Web4C/etc/4creqd
	chmod 500 $HomeDir/Web4C/etc/4creqd
}
#
# Always include source include files
#
	$PRINT "	Installing Includes" >>$FCINSTHIST
	$PRINT "Copying Include Files"
	DirInstall $TopDir/src $HomeDir/src $ROMode "Recurse"
#
# Always include libtestcalc1.d
#
	$PRINT "	Installing libtestcalc1" >>$FCINSTHIST
	$PRINT "Copying libtestcalc1 Files"
	DirInstall $TopDir/libtestcalc1.d $HomeDir/libtestcalc1.d $ROMode "Recurse"
if [ ! -d "$HomeDir/tmp" ]
then
	$PRINT "Creating Tmp Directory"
	mkdir "$HomeDir/tmp"
	chmod $DirMode "$HomeDir/tmp"
	chgrp "$FCGroup" "$HomeDir/tmp"
	chown "$FCOwner" "$HomeDir/tmp"
fi
if [ ! -d "$HomeDir/log" ]
then
	$PRINT "Creating Log Directory"
	mkdir "$HomeDir/log"
	chmod $DirMode "$HomeDir/log"
	chgrp "$FCGroup" "$HomeDir/log"
	chown "$FCOwner" "$HomeDir/log"
fi
genscriptfiles
if [ ! -d "$HomeDir/security" ]
then
	$PRINT "Creating security Directory"
	mkdir $HomeDir/security
	chmod 700 $HomeDir/security
	cd $HomeDir/security
	cp $HomeDir/samples/SecurityConfig .
	cp $HomeDir/samples/AuthClient .
	cp $HomeDir/samples/AuthUser .
	chmod 400 SecurityConfig AuthClient AuthUser
	$PRINT "Generating DH Param File"
#	sh $HomeDir/lib/gendhparam.sh
	$HomeDir/bin/fcgendhparam dhparam.pem 1024 5
	chmod 400 dhparam.pem
	$PRINT "Generating RSA HostKey"
#	sh $HomeDir/lib/genrsa.sh
	$HomeDir/bin/fcgenrsapair key_rsa 1024
	chmod 400 key_rsa
	chmod 444 key_rsa.pub
	mv key_rsa hostkey_rsa
	mv key_rsa.pub hostkey_rsa.pub
fi
if [ -z "$_4CSRVRCONFIG" ]
then
	_4CSRVRCONFIG=$HOME_4C/4CSrvrConfig
	export _4CSRVRCONFIG
fi
if [ -f "$HomeDir/security/serial.txt" ]
then
#
# Update serializion file to current SerialVersion
# This may do nothing and it will not update a file
# that cannot be verified
# A backup of the original is always saved
#
	$FCBinDir/4csp -U -v 0
fi
#yn "Start 4C Server Daemon Now" && /bin/sh "$HomeDir/run4c.sh"
$PRINT "Don't Forget To Verify sec_ddaccess!!!"
$PRINT "***"
$PRINT "*** Check script files for 4cstat calls that need to change to 4cstatd"
$PRINT "***"
instexit 0
